@@ -23,9 +23,8 @@
 #include "config.h"
 #include "conf.h"
 
-// retval indicates need for net bind caps, if possible
 F_NONNULL
-bool udp_sock_setup(dns_thread_t* t);
+void udp_sock_setup(dns_thread_t* t);
 
 F_NONNULL F_NORETURN
 void* dnsio_udp_start(void* thread_asvoid);
