@@ -20,10 +20,13 @@
 #ifndef GDNSD_MISC_PRIV_H
 #define GDNSD_MISC_PRIV_H
 
-#include "gdnsd/compiler.h"
-#include "gdnsd/misc.h"
+#include <gdnsd/compiler.h>
+#include <gdnsd/misc.h>
 
 // Globally initialize meta-prng at daemon startup
 void gdnsd_rand_meta_init(void);
 
+// Kill+Reap pids from gdnsd_register_child_pid()
+void gdnsd_kill_registered_children(void);
+
 #endif // GDNSD_MISC_PRIV_H
