@@ -2,7 +2,7 @@
 #line 1 "./vscf.rl"
 /* Copyright © 2012 Brandon L Black <blblack@gmail.com>
  *
- * This file is part of vscf.
+ * This file is part of gdnsd.
  *
  * vscf is free software: you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -29,8 +29,9 @@
 #include <fcntl.h>
 #include <inttypes.h>
 
-#include "gdnsd/dmn.h"
-#include "gdnsd/vscf.h"
+#include <gdnsd/alloc.h>
+#include <gdnsd/dmn.h>
+#include <gdnsd/vscf.h>
 
 /*
  * The initial size of the read()/fread() buffer.  Note that
@@ -45,7 +46,7 @@
 #define set_err(_epp, _fmt, ...) do { \
     dmn_assert(_epp); \
     if(!*_epp) { \
-        *_epp = malloc(256); \
+        *_epp = xmalloc(256); \
         snprintf(*_epp, 256, _fmt, __VA_ARGS__); \
     } \
 } while(0)
@@ -108,8 +109,8 @@ typedef struct {
     int           cont_stack_top;
     int           cs;
     int           top;
-    unsigned      cont_stack_alloc;
-    unsigned      cs_stack_alloc;
+    int           cont_stack_alloc;
+    int           cs_stack_alloc;
     unsigned      lcount;
     unsigned      cur_klen;
     vscf_data_t*  cont;
@@ -151,7 +152,7 @@ static unsigned djb_hash(const char* k, unsigned klen, const unsigned hash_mask)
 }
 
 static vscf_hash_t* hash_new(void) {
-    vscf_hash_t* h = calloc(1, sizeof(vscf_hash_t));
+    vscf_hash_t* h = xcalloc(1, sizeof(vscf_hash_t));
     h->type = VSCF_HASH_T;
     return h;
 }
@@ -162,7 +163,7 @@ static void hash_grow(vscf_hash_t* h) {
 
     const unsigned old_hash_mask = count2mask(h->child_count);
     const unsigned new_hash_mask = (old_hash_mask << 1) | 1;
-    vscf_hentry_t** new_table = calloc(new_hash_mask + 1, sizeof(vscf_hentry_t*));
+    vscf_hentry_t** new_table = xcalloc(new_hash_mask + 1, sizeof(vscf_hentry_t*));
     for(unsigned i = 0; i <= old_hash_mask; i++) {
         vscf_hentry_t* entry = h->children[i];
         while(entry) {
@@ -187,7 +188,7 @@ static void hash_grow(vscf_hash_t* h) {
     free(h->children);
 
     h->children = new_table;
-    h->ordered = realloc(h->ordered, (new_hash_mask + 1) * sizeof(vscf_hentry_t*));
+    h->ordered = xrealloc(h->ordered, (new_hash_mask + 1) * sizeof(vscf_hentry_t*));
 }
 
 F_NONNULL
@@ -196,8 +197,8 @@ static bool hash_add_val(const char* key, const unsigned klen, vscf_hash_t* h, v
     v->parent = (vscf_data_t*)h;
 
     if(!h->children) {
-        h->children = calloc(2, sizeof(vscf_hentry_t*));
-        h->ordered = malloc(2 * sizeof(vscf_hentry_t*));
+        h->children = xcalloc(2, sizeof(vscf_hentry_t*));
+        h->ordered = xmalloc(2 * sizeof(vscf_hentry_t*));
     }
 
     const unsigned child_mask = count2mask(h->child_count);
@@ -212,9 +213,9 @@ static bool hash_add_val(const char* key, const unsigned klen, vscf_hash_t* h, v
         store_at = &((*store_at)->next);
     }
 
-    vscf_hentry_t* new_hentry = *store_at = calloc(1, sizeof(vscf_hentry_t));
+    vscf_hentry_t* new_hentry = *store_at = xcalloc(1, sizeof(vscf_hentry_t));
     new_hentry->klen = klen;
-    new_hentry->key = malloc(klen + 1);
+    new_hentry->key = xmalloc(klen + 1);
     memcpy(new_hentry->key, key, klen + 1);
     new_hentry->index = h->child_count;
     new_hentry->val = v;
@@ -247,7 +248,7 @@ static bool scnr_hash_add_val(vscf_scnr_t* scnr, vscf_hash_t* h, vscf_data_t* v)
 }
 
 static vscf_array_t* array_new(void) {
-    vscf_array_t* a = calloc(1, sizeof(vscf_array_t));
+    vscf_array_t* a = xcalloc(1, sizeof(vscf_array_t));
     a->type   = VSCF_ARRAY_T;
     return a;
 }
@@ -257,15 +258,15 @@ static void array_add_val(vscf_array_t* a, vscf_data_t* v) {
     dmn_assert(a); dmn_assert(v);
     v->parent = (vscf_data_t*)a;
     unsigned idx = a->len++;
-    a->vals = realloc(a->vals, a->len * sizeof(vscf_data_t*));
+    a->vals = xrealloc(a->vals, a->len * sizeof(vscf_data_t*));
     a->vals[idx] = v;
 }
 
 F_NONNULL
 static vscf_simple_t* simple_new(const char* rval, const unsigned rlen) {
     dmn_assert(rval);
-    vscf_simple_t* s = calloc(1, sizeof(vscf_simple_t));
-    char* storage = malloc(rlen);
+    vscf_simple_t* s = xcalloc(1, sizeof(vscf_simple_t));
+    char* storage = xmalloc(rlen);
     memcpy(storage, rval, rlen);
     s->type   = VSCF_SIMPLE_T;
     s->rlen   = rlen;
@@ -333,11 +334,11 @@ F_NONNULL
 static unsigned unescape_string(char** outp, const char* in, unsigned len) {
     dmn_assert(outp);
     dmn_assert(in);
-    char* out = malloc(len + 1);
+    char* out = xmalloc(len + 1);
     unsigned newlen = len;
     if(len)
-        newlen = dns_unescape((uint8_t*)out, (const uint8_t*)in, len);
-    out = realloc(out, newlen + 1); // downsize
+        newlen = dns_unescape(out, in, len);
+    out = xrealloc(out, newlen + 1); // downsize
     out[newlen] = 0;
     *outp = out;
     return newlen;
@@ -421,7 +422,7 @@ static bool scnr_proc_include(vscf_scnr_t* scnr, const char* end) {
     }
 
     char* inc_parse_err = NULL;
-    vscf_data_t* inc_data = (vscf_data_t*)vscf_scan_filename(final_scan_path, &inc_parse_err);
+    vscf_data_t* inc_data = vscf_scan_filename(final_scan_path, &inc_parse_err);
     if(final_scan_path != input_fn)
         free(final_scan_path);
 
@@ -459,9 +460,8 @@ static bool scnr_proc_include(vscf_scnr_t* scnr, const char* end) {
 }
 
 F_NONNULL
-static void vscf_simple_ensure_val(const vscf_simple_t* cs) {
-    dmn_assert(cs);
-    vscf_simple_t* s = (vscf_simple_t*)cs; // kill const...
+static void vscf_simple_ensure_val(vscf_simple_t* s) {
+    dmn_assert(s);
     if(!s->val)
         s->len = unescape_string(&s->val, s->rval, s->rlen);
 }
@@ -471,8 +471,8 @@ static bool cont_stack_push(vscf_scnr_t* scnr, vscf_data_t* c) {
     dmn_assert(scnr); dmn_assert(c);
     dmn_assert(scnr->cont);
 
-    if(++scnr->cont_stack_top == (signed)scnr->cont_stack_alloc)
-        scnr->cont_stack = realloc(scnr->cont_stack, ++scnr->cont_stack_alloc * sizeof(vscf_data_t*));
+    if(++scnr->cont_stack_top == scnr->cont_stack_alloc)
+        scnr->cont_stack = xrealloc(scnr->cont_stack, ++scnr->cont_stack_alloc * sizeof(vscf_data_t*));
 
     if(!add_to_cur_container(scnr, c))
         return false;
@@ -1298,12 +1298,14 @@ static const int vscf_en_main = 242;
 #line 683 "./vscf.rl"
 
 
-static const vscf_data_t* vscf_scan_fd(const int fd, const char* fn, char** err) {
+static vscf_data_t* vscf_scan_fd(const int fd, const char* fn, char** err) {
     dmn_assert(fd > -1); dmn_assert(fn); dmn_assert(err); dmn_assert(*err == NULL);
 
-    vscf_scnr_t* scnr = calloc(1, sizeof(vscf_scnr_t));
+    (void)vscf_en_main; // silence unused var warning from generated code
+
+    vscf_scnr_t* scnr = xcalloc(1, sizeof(vscf_scnr_t));
     unsigned buf_size = INIT_BUF_SIZE;
-    char* buf = malloc(buf_size);
+    char* buf = xmalloc(buf_size);
     dmn_assert(buf);
 
     scnr->lcount = 1;
@@ -1323,7 +1325,7 @@ static const vscf_data_t* vscf_scan_fd(const int fd, const char* fn, char** err)
             have = scnr->pe - scnr->tstart;
             if(scnr->tstart == buf) {
                 buf_size *= 2;
-                buf = realloc(buf, buf_size);
+                buf = xrealloc(buf, buf_size);
                 dmn_assert(buf);
             }
             else {
@@ -1346,7 +1348,7 @@ static const vscf_data_t* vscf_scan_fd(const int fd, const char* fn, char** err)
             scnr->eof = scnr->pe;
 
         
-#line 1350 "./vscf.c"
+#line 1352 "./vscf.c"
 	{
 	int _klen;
 	unsigned int _trans;
@@ -1459,9 +1461,9 @@ _match:
         if(!cont_stack_push(scnr, (vscf_data_t*)array_new()))
             {(     scnr->p)++; goto _out; }
         {
-                if(scnr->top == (signed)scnr->cs_stack_alloc)
+                if(scnr->top == scnr->cs_stack_alloc)
                     scnr->cs_stack
-                        = realloc(scnr->cs_stack,
+                        = xrealloc(scnr->cs_stack,
                             ++scnr->cs_stack_alloc * sizeof(int));
             {( scnr->cs_stack)[(   scnr->top)++] = (    scnr->cs); (    scnr->cs) = 120; goto _again;}}
     }
@@ -1472,9 +1474,9 @@ _match:
         if(!cont_stack_push(scnr, (vscf_data_t*)hash_new()))
             {(     scnr->p)++; goto _out; }
         {
-                if(scnr->top == (signed)scnr->cs_stack_alloc)
+                if(scnr->top == scnr->cs_stack_alloc)
                     scnr->cs_stack
-                        = realloc(scnr->cs_stack,
+                        = xrealloc(scnr->cs_stack,
                             ++scnr->cs_stack_alloc * sizeof(int));
             {( scnr->cs_stack)[(   scnr->top)++] = (    scnr->cs); (    scnr->cs) = 170; goto _again;}}
     }
@@ -1518,7 +1520,7 @@ _match:
             {(     scnr->p)++; goto _out; }
     }
 	break;
-#line 1522 "./vscf.c"
+#line 1524 "./vscf.c"
 		}
 	}
 
@@ -1553,7 +1555,7 @@ _again:
 #line 566 "./vscf.rl"
 	{ scnr->lcount++; }
 	break;
-#line 1557 "./vscf.c"
+#line 1559 "./vscf.c"
 		}
 	}
 	}
@@ -1561,7 +1563,7 @@ _again:
 	_out: {}
 	}
 
-#line 746 "./vscf.rl"
+#line 748 "./vscf.rl"
 
 
         if(scnr->cs == vscf_error) {
@@ -1591,7 +1593,7 @@ _again:
         free(scnr->cs_stack);
     free(buf);
 
-    const vscf_data_t* retval;
+    vscf_data_t* retval;
 
     if(*err) {
         if(scnr->cont_stack_top == -1)
@@ -1602,7 +1604,7 @@ _again:
     }
     else {
         dmn_assert(scnr->cont_stack_top == -1);
-        retval = (const vscf_data_t*)scnr->cont; // outermost container
+        retval = scnr->cont; // outermost container
     }
 
     if(scnr->cont_stack)
@@ -1616,7 +1618,7 @@ _again:
 /*** Public API functions ***/
 /****************************/
 
-const vscf_data_t* vscf_scan_filename(const char* fn, char** err) {
+vscf_data_t* vscf_scan_filename(const char* fn, char** err) {
     dmn_assert(fn); dmn_assert(err);
     *err = NULL;
 
@@ -1626,27 +1628,27 @@ const vscf_data_t* vscf_scan_filename(const char* fn, char** err) {
         return NULL;
     }
 
-    const vscf_data_t* retval = vscf_scan_fd(fd, fn, err);
+    vscf_data_t* retval = vscf_scan_fd(fd, fn, err);
     close(fd);
     return retval;
 }
 
-void vscf_destroy(const vscf_data_t* d) { val_destroy((vscf_data_t*)d); }
+void vscf_destroy(vscf_data_t* d) { val_destroy(d); }
 
 vscf_type_t vscf_get_type(const vscf_data_t* d) { dmn_assert(d); return d->type; }
 bool vscf_is_simple(const vscf_data_t* d) { dmn_assert(d); return d->type == VSCF_SIMPLE_T; }
 bool vscf_is_array(const vscf_data_t* d) { dmn_assert(d); return d->type == VSCF_ARRAY_T; }
 bool vscf_is_hash(const vscf_data_t* d) { dmn_assert(d); return d->type == VSCF_HASH_T; }
 bool vscf_is_root(const vscf_data_t* d) { dmn_assert(d); return d->parent == NULL; }
-const vscf_data_t* vscf_get_parent(const vscf_data_t* d) { dmn_assert(d); return d->parent; }
+vscf_data_t* vscf_get_parent(const vscf_data_t* d) { dmn_assert(d); return d->parent; }
 
-unsigned vscf_simple_get_len(const vscf_data_t* d) {
+unsigned vscf_simple_get_len(vscf_data_t* d) {
     dmn_assert(d); dmn_assert(vscf_is_simple(d));
     vscf_simple_ensure_val(&d->simple);
     return d->simple.len;
 }
 
-const char* vscf_simple_get_data(const vscf_data_t* d) {
+const char* vscf_simple_get_data(vscf_data_t* d) {
     dmn_assert(d); dmn_assert(vscf_is_simple(d));
     vscf_simple_ensure_val(&d->simple);
     return d->simple.val;
@@ -1659,7 +1661,7 @@ unsigned vscf_array_get_len(const vscf_data_t* d) {
     return d->array.len;
 }
 
-const vscf_data_t* vscf_array_get_data(const vscf_data_t* d, unsigned idx) {
+vscf_data_t* vscf_array_get_data(vscf_data_t* d, unsigned idx) {
     dmn_assert(d);
     if(d->type != VSCF_ARRAY_T) {
         if(idx) return NULL;
@@ -1674,7 +1676,7 @@ unsigned vscf_hash_get_len(const vscf_data_t* d) {
     return d->hash.child_count;
 }
 
-const vscf_data_t* vscf_hash_get_data_bykey(const vscf_data_t* d, const char* key, unsigned klen, bool set_mark) {
+vscf_data_t* vscf_hash_get_data_bykey(const vscf_data_t* d, const char* key, unsigned klen, bool set_mark) {
     dmn_assert(d); dmn_assert(vscf_is_hash(d));
     dmn_assert(key);
     if(d->hash.child_count) {
@@ -1702,10 +1704,10 @@ const char* vscf_hash_get_key_byindex(const vscf_data_t* d, unsigned idx, unsign
     return rv;
 }
 
-const vscf_data_t* vscf_hash_get_data_byindex(const vscf_data_t* d, unsigned idx) {
+vscf_data_t* vscf_hash_get_data_byindex(const vscf_data_t* d, unsigned idx) {
     dmn_assert(d); dmn_assert(vscf_is_hash(d));
     if(idx >= d->hash.child_count) return NULL;
-    const vscf_data_t* rv = d->hash.ordered[idx]->val;
+    vscf_data_t* rv = d->hash.ordered[idx]->val;
     dmn_assert(rv);
     return rv;
 }
@@ -1738,6 +1740,17 @@ void vscf_hash_iterate(const vscf_data_t* d, bool ignore_mark, vscf_hash_iter_cb
     }
 }
 
+void vscf_hash_iterate_const(const vscf_data_t* d, bool ignore_mark, vscf_hash_iter_const_cb_t f, const void* data) {
+    dmn_assert(d); dmn_assert(vscf_is_hash(d));
+    dmn_assert(f);
+    for(unsigned i = 0; i < d->hash.child_count; i++) {
+        const vscf_hentry_t* hentry = d->hash.ordered[i];
+        if(!ignore_mark || !hentry->marked)
+            if(!f(hentry->key, hentry->klen, hentry->val, data))
+                return;
+    }
+}
+
 void vscf_hash_sort(const vscf_data_t* d, vscf_key_cmp_cb_t f) {
     dmn_assert(d); dmn_assert(vscf_is_hash(d));
     dmn_assert(f);
@@ -1748,7 +1761,7 @@ void vscf_hash_sort(const vscf_data_t* d, vscf_key_cmp_cb_t f) {
         d->hash.ordered[i]->index = i;
 }
 
-bool vscf_simple_get_as_ulong(const vscf_data_t* d, unsigned long* out) {
+bool vscf_simple_get_as_ulong(vscf_data_t* d, unsigned long* out) {
     dmn_assert(d); dmn_assert(vscf_is_simple(d));
     dmn_assert(out);
     vscf_simple_ensure_val(&d->simple);
@@ -1766,7 +1779,7 @@ bool vscf_simple_get_as_ulong(const vscf_data_t* d, unsigned long* out) {
     return true;
 }
 
-bool vscf_simple_get_as_long(const vscf_data_t* d, long* out) {
+bool vscf_simple_get_as_long(vscf_data_t* d, long* out) {
     dmn_assert(d); dmn_assert(vscf_is_simple(d));
     dmn_assert(out);
     vscf_simple_ensure_val(&d->simple);
@@ -1784,7 +1797,7 @@ bool vscf_simple_get_as_long(const vscf_data_t* d, long* out) {
     return true;
 }
 
-bool vscf_simple_get_as_double(const vscf_data_t* d, double* out) {
+bool vscf_simple_get_as_double(vscf_data_t* d, double* out) {
     dmn_assert(d); dmn_assert(vscf_is_simple(d));
     dmn_assert(out);
     vscf_simple_ensure_val(&d->simple);
@@ -1802,7 +1815,7 @@ bool vscf_simple_get_as_double(const vscf_data_t* d, double* out) {
     return true;
 }
 
-bool vscf_simple_get_as_bool(const vscf_data_t* d, bool* out) {
+bool vscf_simple_get_as_bool(vscf_data_t* d, bool* out) {
     dmn_assert(d); dmn_assert(vscf_is_simple(d));
     dmn_assert(out);
     vscf_simple_ensure_val(&d->simple);
@@ -1831,7 +1844,7 @@ bool vscf_simple_get_as_bool(const vscf_data_t* d, bool* out) {
 dname_status_t vscf_simple_get_as_dname(const vscf_data_t* d, uint8_t* dname) {
     dmn_assert(d); dmn_assert(vscf_is_simple(d));
     dmn_assert(dname);
-    return dname_from_string(dname, (const uint8_t*)d->simple.rval, d->simple.rlen);
+    return dname_from_string(dname, d->simple.rval, d->simple.rlen);
 }
 
 vscf_data_t* vscf_hash_new(void) { return (vscf_data_t*)hash_new(); }
@@ -1886,7 +1899,7 @@ bool vscf_hash_bequeath_all(const vscf_data_t* src, const char* k, const bool ma
     if(src_val) {
         const unsigned src_len = vscf_hash_get_len(src);
         for(unsigned i = 0; i < src_len; i++) {
-            vscf_data_t* child_val = (vscf_data_t*)vscf_hash_get_data_byindex(src, i);
+            vscf_data_t* child_val = vscf_hash_get_data_byindex(src, i);
             if(vscf_is_hash(child_val) && (!skip_marked || !src->hash.ordered[i]->marked))
                 if(!vscf_hash_get_data_bystringkey(child_val, k, false))
                     vscf_hash_add_val(k, strlen(k), child_val, vscf_clone(src_val, false));
