@@ -21,14 +21,22 @@
 #define GDSND_STATIO_H
 
 #include "config.h"
-#include "gdnsd/compiler.h"
+#include <gdnsd/compiler.h>
+#include <stdbool.h>
 #include <ev.h>
 
-void statio_log_stats(void);
-void statio_log_uptime(void);
-
-F_NONNULL void statio_start(struct ev_loop* statio_loop);
 void statio_init(void);
+void statio_bind_socks(void);
+bool statio_check_socks(bool soft);
+
+F_NONNULL
+void statio_start(struct ev_loop* statio_loop);
+
+// main thread calls this to issue final stats output
+void statio_final_stats(void);
+
+// main thread calls this to wait on completion of the above
+void statio_final_stats_wait(void);
 
 #endif // GDSND_STATIO_H
 
