@@ -18,8 +18,9 @@
  */
 
 #include "config.h"
-#include "gdnsd/compiler.h"
-#include "gdnsd/log.h"
+#include <gdnsd/alloc.h>
+#include <gdnsd/compiler.h>
+#include <gdnsd/log.h>
 
 #include "extmon_comms.h"
 
@@ -31,9 +32,15 @@
 #include <sys/types.h>
 #include <sys/wait.h>
 #include <signal.h>
+#include <time.h>
 
 #include <ev.h>
 
+// hopefully everyone defines this
+#ifndef NSIG
+#  define NSIG 100
+#endif
+
 typedef struct {
     extmon_cmd_t* cmd;
     ev_timer* interval_timer;
@@ -46,13 +53,16 @@ typedef struct {
 static unsigned num_mons = 0;
 static mon_t* mons = NULL;
 
-F_NONNULL
-static void syserr_for_ev(const char* msg) { dmn_assert(msg); log_fatal("%s: %s", msg, logf_errno()); }
+F_NONNULL F_NORETURN
+static void syserr_for_ev(const char* msg) { dmn_assert(msg); log_fatal("%s: %s", msg, dmn_logf_errno()); }
 
 static int plugin_read_fd = -1;
 static int plugin_write_fd = -1;
-static ev_io* plugin_read_watcher = NULL;
 static ev_io* plugin_write_watcher = NULL;
+static ev_signal* sigterm_watcher = NULL;
+static ev_signal* sigint_watcher = NULL;
+static ev_signal* sighup_watcher = NULL;
+static int killed_by = 0;
 
 /*************************************************************************/
 // This implements a simple unidirectional FIFO queue for buffering
@@ -60,15 +70,15 @@ static ev_io* plugin_write_watcher = NULL;
 // The queue is actually an array with moving head pointer that
 //   wraps circularly, and is resized via re-allocation when necc.
 
-uint32_t* sendq = NULL;
-unsigned sendq_alloc = 0;
-unsigned sendq_len = 0;
-unsigned sendq_head = 0;
+static uint32_t* sendq = NULL;
+static unsigned sendq_alloc = 0;
+static unsigned sendq_len = 0;
+static unsigned sendq_head = 0;
 
 // must be power of 2!
 #define SENDQ_INITSIZE 16
 static void sendq_init(void) {
-    sendq = malloc(SENDQ_INITSIZE * sizeof(uint32_t));
+    sendq = xmalloc(SENDQ_INITSIZE * sizeof(uint32_t));
     sendq_alloc = SENDQ_INITSIZE;
 }
 
@@ -82,7 +92,7 @@ static void sendq_enq(uint32_t new_data) {
         // buffer too small, upsize first (we never downsize)
         const unsigned old_mask = sendq_alloc - 1;
         sendq_alloc <<= 1;
-        uint32_t* newq = malloc(sendq_alloc * sizeof(uint32_t));
+        uint32_t* newq = xmalloc(sendq_alloc * sizeof(uint32_t));
         for(unsigned i = 0; i < sendq_len; i++)
             newq[i] = sendq[(sendq_head + i) & old_mask];
         newq[sendq_len] = new_data;
@@ -131,8 +141,10 @@ static void mon_timeout_cb(struct ev_loop* loop, ev_timer* w, int revents V_UNUS
     //   and restart the child watcher for a new child, effectively
     //   giving up on waitpid() of this child.  Not much else we
     //   could do in that case anyways.
-    sendq_enq(emc_encode_mon(this_mon->cmd->idx, true));
-    ev_io_start(loop, plugin_write_watcher);
+    if(!killed_by) {
+        sendq_enq(emc_encode_mon(this_mon->cmd->idx, true));
+        ev_io_start(loop, plugin_write_watcher);
+    }
     this_mon->result_pending = false;
 }
 
@@ -161,8 +173,10 @@ static void mon_child_cb(struct ev_loop* loop, ev_child* w, int revents V_UNUSED
     // If timeout already sent a failure, don't double-send
     //   here when we reap the SIGKILL'd child
     if(this_mon->result_pending) {
-        sendq_enq(emc_encode_mon(this_mon->cmd->idx, failed));
-        ev_io_start(loop, plugin_write_watcher);
+        if(!killed_by) {
+            sendq_enq(emc_encode_mon(this_mon->cmd->idx, failed));
+            ev_io_start(loop, plugin_write_watcher);
+        }
         this_mon->result_pending = false;
     }
 }
@@ -173,18 +187,48 @@ static void mon_interval_cb(struct ev_loop* loop, ev_timer* w, int revents V_UNU
     mon_t* this_mon = w->data;
     dmn_assert(!this_mon->result_pending);
 
+    // Before forking, block all signals and save the old mask
+    //   to avoid a race condition where local sighandlers execute
+    //   in the child between fork and exec().
+    sigset_t all_sigs;
+    sigfillset(&all_sigs);
+    sigset_t saved_mask;
+    sigemptyset(&saved_mask);
+    if(pthread_sigmask(SIG_SETMASK, &all_sigs, &saved_mask))
+        log_fatal("pthread_sigmask() failed");
+
     this_mon->cmd_pid = fork();
     if(this_mon->cmd_pid == -1)
-        log_fatal("fork() failed: %s", dmn_strerror(errno));
+        log_fatal("fork() failed: %s", dmn_logf_strerror(errno));
 
     if(!this_mon->cmd_pid) { // child
+        // reset all signal handlers to default before unblocking
+        struct sigaction defaultme;
+        sigemptyset(&defaultme.sa_mask);
+        defaultme.sa_handler = SIG_DFL;
+        defaultme.sa_flags = 0;
+
+        // we really don't care about error retvals here
+        for(unsigned i = 0; i < NSIG; i++)
+            (void)sigaction(i, &defaultme, NULL);
+
+        // unblock all
+        sigset_t no_sigs;
+        sigemptyset(&no_sigs);
+        if(pthread_sigmask(SIG_SETMASK, &no_sigs, NULL))
+            log_fatal("pthread_sigmask() failed");
+
         // technically, we could go ahead and close off stdout/stderr
         //   here for the "startfg" case, but why bother?  If the user
         //   is debugging via startfg they might want to see this crap anyways.
         execv(this_mon->cmd->args[0], (char* const *)this_mon->cmd->args);
-        log_fatal("execv(%s, ...) failed: %s", this_mon->cmd->args[0], dmn_strerror(errno));
+        log_fatal("execv(%s, ...) failed: %s", this_mon->cmd->args[0], dmn_logf_strerror(errno));
     }
 
+    // restore previous signal mask from before fork in parent
+    if(pthread_sigmask(SIG_SETMASK, &saved_mask, NULL))
+        log_fatal("pthread_sigmask() failed");
+
     this_mon->result_pending = true;
     ev_timer_set(this_mon->cmd_timeout, this_mon->cmd->timeout, 0);
     ev_timer_start(loop, this_mon->cmd_timeout);
@@ -195,12 +239,13 @@ static void mon_interval_cb(struct ev_loop* loop, ev_timer* w, int revents V_UNU
 static void plugin_write_cb(struct ev_loop* loop, ev_io* w, int revents V_UNUSED) {
     dmn_assert(loop); dmn_assert(w); dmn_assert(revents == EV_WRITE);
 
+    dmn_assert(plugin_write_fd > -1);
     while(!sendq_empty()) {
         const uint32_t data = sendq_deq_peek();
         int rv = write(plugin_write_fd, &data, 4);
         if(rv != 4) {
             if(rv < 0) {
-                if(errno == EAGAIN)
+                if(errno == EAGAIN || errno == EWOULDBLOCK)
                     return; // pipe full, wait for more libev notification of write-ready
                 else if(errno == EINTR)
                     continue; // try this write again immediately
@@ -220,18 +265,57 @@ static void plugin_write_cb(struct ev_loop* loop, ev_io* w, int revents V_UNUSED
         sendq_deq_commit();
     }
     ev_io_stop(loop, w); // queue now empty
+
+    if(killed_by) { // we've sent our final message, close
+        close(plugin_write_fd);
+        plugin_write_fd = -1;
+    }
+}
+
+F_NONNULL
+static void die_gracefully(struct ev_loop* loop) {
+    dmn_assert(loop);
+
+    dmn_assert(killed_by);
+    static bool done_once = false;
+    if(!done_once) { // avoid repetition
+        done_once = true;
+        // send friendly death message to plugin
+        sendq_enq(emc_encode_exit());
+        ev_io_start(loop, plugin_write_watcher);
+        // kill interval timers for future invocations
+        //   and immediately clamp the remaining timeout
+        //   for any running commands to 2.0s.
+        for(unsigned i = 0; i < num_mons; i++) {
+            ev_timer_stop(loop, mons[i].interval_timer);
+            if(ev_is_active(mons[i].cmd_timeout)) {
+                if(ev_timer_remaining(loop, mons[i].cmd_timeout) > 2.0) {
+                    ev_timer_stop(loop, mons[i].cmd_timeout);
+                    ev_timer_set(mons[i].cmd_timeout, 2.0, 0.);
+                    ev_timer_start(loop, mons[i].cmd_timeout);
+                }
+            }
+        }
+    }
+}
+
+F_NONNULL
+static void sig_cb(struct ev_loop* loop, ev_signal* w, int revents V_UNUSED) {
+    dmn_assert(loop); dmn_assert(w); dmn_assert(revents == EV_SIGNAL);
+    switch(w->signum) {
+        case SIGINT:
+        case SIGTERM:
+        case SIGHUP:
+            killed_by = w->signum;
+            dmn_log_info("Got terminal signal %i, starting graceful exit", killed_by);
+            die_gracefully(loop);
+            break;
+        default:
+            dmn_assert(0);
+    }
 }
 
 int main(int argc, char** argv) {
-    dmn_init_log("gdnsd_extmon_helper", true);
-
-    // start up syslog IFF it appears the daemon
-    //   was *not* started via "startfg".  Regular
-    //   start/restart would have /dev/null'd the
-    //   standard descriptors before forking us off.
-    if(!isatty(0))
-        dmn_start_syslog();
-
     // Bail out early if we don't have the right argument
     //   count, and try to tell the user not to run us
     //   if stderr happens to be hooked up to a terminal
@@ -240,21 +324,26 @@ int main(int argc, char** argv) {
         exit(99);
     }
 
-    // open stderr logging connection using passed fd
-    dmn_log_set_alt_stderr(atoi(argv[2]));
+    bool debug = false;
+    if(!strcmp(argv[1], "Y"))
+        debug = true;
+
+    bool use_syslog = false;
+    if(!strcmp(argv[2], "S"))
+        use_syslog = true;
+
+    dmn_init1(debug, true, use_syslog, "gdnsd_extmon_helper");
 
     // regardless, we seal off stdin now.  We don't need it,
     //   and this way we don't have to deal with it when
     //   execv()-ing child commands later.
     if(!freopen("/dev/null", "r", stdin))
-        dmn_log_fatal("Cannot open /dev/null: %s", dmn_strerror(errno));
+        dmn_log_fatal("Cannot open /dev/null: %s", dmn_logf_strerror(errno));
 
-    if(!strcmp(argv[1], "Y"))
-        dmn_set_debug(true);
-    else if(!strcmp(argv[1], "N"))
-        dmn_set_debug(false);
-    else
-        log_fatal("Invalid debug argument on cmdline: '%s'!", argv[1]);
+    // Also unconditionally unset NOTIFY_SOCKET here so that children
+    //   don't get any ideas about talking to systemd on our behalf.
+    //   (we're done using it in this process for libdmn stuff at this point)
+    unsetenv("NOTIFY_SOCKET");
 
     // these are the main communication pipes to the daemon/plugin
     plugin_read_fd = atoi(argv[3]);
@@ -264,13 +353,6 @@ int main(int argc, char** argv) {
         || plugin_write_fd < 3 || plugin_write_fd > 1000)
         log_fatal("Invalid pipe descriptors!");
 
-    // CLOEXEC the direct lines to the main plugin/daemon,
-    //   so that child scripts can't screw with them.
-    if(fcntl(plugin_read_fd, F_SETFD, FD_CLOEXEC))
-        log_fatal("Failed to set FD_CLOEXEC on plugin read fd: %s", dmn_strerror(errno));
-    if(fcntl(plugin_write_fd, F_SETFD, FD_CLOEXEC))
-        log_fatal("Failed to set FD_CLOEXEC on plugin write fd: %s", dmn_strerror(errno));
-
     if(emc_read_exact(plugin_read_fd, "HELO"))
         log_fatal("Failed to read HELO from plugin");
     if(emc_write_string(plugin_write_fd, "HELO_ACK", 8))
@@ -283,7 +365,7 @@ int main(int argc, char** argv) {
     num_mons = ((unsigned)ccount_buf[5] << 8) + ccount_buf[6];
     if(!num_mons)
         log_fatal("Received command count of zero from plugin");
-    mons = calloc(num_mons, sizeof(mon_t));
+    mons = xcalloc(num_mons, sizeof(mon_t));
 
     if(emc_write_string(plugin_write_fd, "CMDS_ACK", 8))
         log_fatal("Failed to write CMDS_ACK to plugin");
@@ -310,7 +392,11 @@ int main(int argc, char** argv) {
     // done with the serial setup, close the readpipe and go nonblocking on write for eventloop...
     close(plugin_read_fd);
     if(unlikely(fcntl(plugin_write_fd, F_SETFL, (fcntl(plugin_write_fd, F_GETFL, 0)) | O_NONBLOCK) == -1))
-        log_fatal("Failed to set O_NONBLOCK on pipe: %s", logf_errno());
+        log_fatal("Failed to set O_NONBLOCK on pipe: %s", dmn_logf_errno());
+
+    // CLOEXEC the write fd so child scripts can't mess with it
+    if(fcntl(plugin_write_fd, F_SETFD, FD_CLOEXEC))
+        log_fatal("Failed to set FD_CLOEXEC on plugin write fd: %s", dmn_logf_strerror(errno));
 
     // init results-sending queue
     sendq_init();
@@ -321,12 +407,23 @@ int main(int argc, char** argv) {
     // Construct the default loop for the main thread
     struct ev_loop* def_loop = ev_default_loop(EVFLAG_AUTO);
     if(!def_loop) log_fatal("Could not initialize the default libev loop");
-    ev_set_timeout_collect_interval(def_loop, 0.1);
-    ev_set_io_collect_interval(def_loop, 0.01);
+
+    // Catch SIGINT/TERM/HUP, and do not let them prevent loop exit
+    sigterm_watcher = xmalloc(sizeof(ev_signal));
+    sigint_watcher = xmalloc(sizeof(ev_signal));
+    sighup_watcher = xmalloc(sizeof(ev_signal));
+    ev_signal_init(sigterm_watcher, sig_cb, SIGTERM);
+    ev_signal_init(sigint_watcher, sig_cb, SIGINT);
+    ev_signal_init(sighup_watcher, sig_cb, SIGHUP);
+    ev_signal_start(def_loop, sigterm_watcher);
+    ev_signal_start(def_loop, sigint_watcher);
+    ev_signal_start(def_loop, sighup_watcher);
+    ev_unref(def_loop);
+    ev_unref(def_loop);
+    ev_unref(def_loop);
 
     // set up primary read/write watchers on the pipe to the daemon's plugin
-    plugin_read_watcher = malloc(sizeof(ev_io));
-    plugin_write_watcher = malloc(sizeof(ev_io));
+    plugin_write_watcher = xmalloc(sizeof(ev_io));
     ev_io_init(plugin_write_watcher, plugin_write_cb, plugin_write_fd, EV_WRITE);
     ev_set_priority(plugin_write_watcher, 1);
 
@@ -334,7 +431,7 @@ int main(int argc, char** argv) {
     //   for the daemon's monitoring init cycle, then repeating every interval.
     for(unsigned i = 0; i < num_mons; i++) {
         mon_t* this_mon = &mons[i];
-        this_mon->interval_timer = malloc(sizeof(ev_timer));
+        this_mon->interval_timer = xmalloc(sizeof(ev_timer));
         ev_timer_init(this_mon->interval_timer, mon_interval_cb, 0., this_mon->cmd->interval);
         this_mon->interval_timer->data = this_mon;
         ev_set_priority(this_mon->interval_timer, 0);
@@ -342,29 +439,60 @@ int main(int argc, char** argv) {
 
         // initialize the other watchers in the mon_t here as well,
         //   but do not start them (the interval callback starts them each interval)
-        this_mon->cmd_timeout = malloc(sizeof(ev_timer));
+        this_mon->cmd_timeout = xmalloc(sizeof(ev_timer));
         ev_timer_init(this_mon->cmd_timeout, mon_timeout_cb, 0, 0);
         ev_set_priority(this_mon->cmd_timeout, -1);
         this_mon->cmd_timeout->data = this_mon;
 
-        this_mon->child_watcher = malloc(sizeof(ev_child));
+        this_mon->child_watcher = xmalloc(sizeof(ev_child));
         ev_child_init(this_mon->child_watcher, mon_child_cb, 0, 0);
         this_mon->child_watcher->data = this_mon;
     }
 
     log_info("gdnsd_extmon_helper running");
-
-    // shut off stderr output from here out...
-    dmn_log_close_alt_stderr();
-
     ev_run(def_loop, 0);
-    log_info("gdnsd_extmon_helper terminating");
 
-    // kill -9 on any extant child procs
-    for(unsigned i = 0; i < num_mons; i++)
-        if(mons[i].cmd_pid)
+    // graceful shutdown should have cleared out children, but
+    // the hard kill/wait below is for (a) ungraceful shutdown
+    // on unexpected pipe close and (b) anything else going wrong
+    // during graceful shutdown.
+    bool needs_wait = false;
+    for(unsigned i = 0; i < num_mons; i++) {
+        if(mons[i].cmd_pid) {
+            log_debug("not-so-graceful shutdown: sending SIGKILL to %li", (long)mons[i].cmd_pid);
             kill(mons[i].cmd_pid, SIGKILL);
+            needs_wait = true;
+        }
+    }
+
+    if(needs_wait) {
+        unsigned i = 500; // 5s for OS to give us all the SIGKILL'd zombies
+        while(i--) {
+            pid_t wprv = waitpid(-1, NULL, WNOHANG);
+            if(wprv < 0) {
+                if(errno == ECHILD)
+                    break;
+                else
+                    log_fatal("waitpid(-1, NULL, WNOHANG) failed: %s", dmn_logf_errno());
+            }
+            if(wprv)
+                log_debug("not-so-graceful shutdown: waitpid reaped %li", (long)wprv);
+            const struct timespec ms_10 = { 0, 10000000 };
+            nanosleep(&ms_10, NULL);
+        }
+    }
 
     // Bye!
-    exit(0);
+    if(killed_by) {
+        log_info("gdnsd_extmon_helper exiting gracefully due to signal %i", killed_by);
+#ifdef COVERTEST_EXIT
+        exit(0);
+#else
+        raise(killed_by);
+#endif
+    }
+    else {
+        log_info("gdnsd_extmon_helper exiting un-gracefully");
+        exit(42);
+    }
 }
