@@ -1,6 +1,6 @@
 /* Copyright © 2012 Brandon L Black <blblack@gmail.com>
  *
- * This file is part of gdnsd-plugin-geoip.
+ * This file is part of gdnsd.
  *
  * gdnsd-plugin-geoip is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -33,7 +33,4 @@ fips_t* fips_init(const char* pathname);
 F_PURE F_NONNULL
 const char* fips_lookup(const fips_t* fips, const uint32_t key);
 
-F_NONNULL
-void fips_destroy(fips_t* fips);
-
 #endif // FIPS104_H
