@@ -1,6 +1,6 @@
 /* Copyright © 2012 Brandon L Black <blblack@gmail.com>
  *
- * This file is part of gdnsd-plugin-geoip.
+ * This file is part of gdnsd.
  *
  * gdnsd-plugin-geoip is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,6 +30,7 @@
 #include <sys/stat.h>
 #include <sys/mman.h>
 
+#include <gdnsd/alloc.h>
 #include <gdnsd/dmn.h>
 #include <gdnsd/log.h>
 #include <gdnsd/vscf.h>
@@ -270,7 +271,7 @@ static unsigned region_get_dclist(const geoip_db_t* db, const unsigned offset) {
 }
 
 F_NONNULL
-static unsigned city_get_dclist(const geoip_db_t* db, unsigned int offs) {
+static unsigned city_get_dclist(const geoip_db_t* db, unsigned offs) {
     dmn_assert(db); dmn_assert(offs >= db->base);
 
     char locstr[256];
@@ -403,7 +404,7 @@ static unsigned get_dclist_cached(geoip_db_t* db, const unsigned offset) {
     }
 
     unsigned dclist = db->dclist_get_func(db, offset);
-    db->offset_cache[ndx] = realloc(db->offset_cache[ndx], sizeof(offset_cache_item_t) * (bucket_size + 2));
+    db->offset_cache[ndx] = xrealloc(db->offset_cache[ndx], sizeof(offset_cache_item_t) * (bucket_size + 2));
     dmn_assert(db->offset_cache[ndx]);
     db->offset_cache[ndx][bucket_size].offset = offset;
     db->offset_cache[ndx][bucket_size].dclist = dclist;
@@ -478,12 +479,12 @@ static bool geoip_db_close(geoip_db_t* db) {
     if(db->fd != -1) {
         if(db->data) {
             if(-1 == munmap(db->data, db->size)) {
-                log_err("plugin_geoip: munmap() of '%s' failed: %s", logf_pathname(db->pathname), logf_errno());
+                log_err("plugin_geoip: munmap() of '%s' failed: %s", db->pathname, dmn_logf_errno());
                 rv = true;
             }
         }
         if(close(db->fd) == -1) {
-            log_err("plugin_geoip: close() of '%s' failed: %s", logf_pathname(db->pathname), logf_errno());
+            log_err("plugin_geoip: close() of '%s' failed: %s", db->pathname, dmn_logf_errno());
             rv = true;
         }
     }
@@ -499,7 +500,7 @@ F_NONNULLX(1,2,3)
 static geoip_db_t* geoip_db_open(const char* pathname, const char* map_name, dclists_t* dclists, const dcmap_t* dcmap, const fips_t* fips, const gdgeoip_v4o_t v4o_flag, const bool city_auto_mode, const bool city_no_region) {
     dmn_assert(pathname); dmn_assert(map_name); dmn_assert(dclists);
 
-    geoip_db_t* db = calloc(1, sizeof(geoip_db_t));
+    geoip_db_t* db = xcalloc(1, sizeof(geoip_db_t));
     db->fd = -1;
     db->pathname = pathname;
     db->map_name = map_name;
@@ -510,14 +511,14 @@ static geoip_db_t* geoip_db_open(const char* pathname, const char* map_name, dcl
     db->city_no_region = city_no_region;
 
     if((db->fd = open(pathname, O_RDONLY)) == -1) {
-        log_err("plugin_geoip: map '%s': Cannot open '%s' for reading: %s", map_name, logf_pathname(pathname), logf_errno());
+        log_err("plugin_geoip: map '%s': Cannot open '%s' for reading: %s", map_name, pathname, dmn_logf_errno());
         geoip_db_close(db);
         return NULL;
     }
 
     struct stat db_stat;
     if(fstat(db->fd, &db_stat) == -1) {
-        log_err("plugin_geoip: map '%s': Cannot fstat '%s': %s", map_name, logf_pathname(pathname), logf_errno());
+        log_err("plugin_geoip: map '%s': Cannot fstat '%s': %s", map_name, pathname, dmn_logf_errno());
         geoip_db_close(db);
         return NULL;
     }
@@ -529,14 +530,14 @@ static geoip_db_t* geoip_db_open(const char* pathname, const char* map_name, dcl
     //   to a single countryid, plus the requisite 0xFFFFFF
     //   end marker.
     if(db->size < 9) {
-        log_err("plugin_geoip: map '%s': GeoIP database '%s' too small", map_name, logf_pathname(pathname));
+        log_err("plugin_geoip: map '%s': GeoIP database '%s' too small", map_name, pathname);
         geoip_db_close(db);
         return NULL;
     }
 
     if((db->data = mmap(NULL, db->size, PROT_READ, MAP_SHARED, db->fd, 0)) == MAP_FAILED) {
         db->data = 0;
-        log_err("plugin_geoip: map '%s': Failed to mmap GeoIP DB '%s': %s", map_name, logf_pathname(pathname), logf_errno());
+        log_err("plugin_geoip: map '%s': Failed to mmap GeoIP DB '%s': %s", map_name, pathname, dmn_logf_errno());
         geoip_db_close(db);
         return NULL;
     }
@@ -570,7 +571,7 @@ static geoip_db_t* geoip_db_open(const char* pathname, const char* map_name, dcl
             case GEOIP_CITY_EDITION_REV1:
                 break;
             default:
-                log_err("plugin_geoip: map '%s': GeoIP DB '%s' is not a City-level database and this map uses auto_dc_coords", map_name, logf_pathname(db->pathname));
+                log_err("plugin_geoip: map '%s': GeoIP DB '%s' is not a City-level database and this map uses auto_dc_coords", map_name, db->pathname);
                 geoip_db_close(db);
                 return NULL;
         }
@@ -613,18 +614,18 @@ static geoip_db_t* geoip_db_open(const char* pathname, const char* map_name, dcl
             break;
 
         default:
-            log_err("plugin_geoip: map '%s': GeoIP DB '%s': Unrecognized DB type %i", map_name, logf_pathname(db->pathname), db->type);
+            log_err("plugin_geoip: map '%s': GeoIP DB '%s': Unrecognized DB type %i", map_name, db->pathname, db->type);
             geoip_db_close(db);
             return NULL;
     }
 
     if((v4o_flag == V4O_PRIMARY) && !db->ipv6) {
-        log_err("plugin_geoip: map '%s': Primary GeoIP DB '%s' is not an IPv6 database and this map uses geoip_v4_overlay", map_name, logf_pathname(db->pathname));
+        log_err("plugin_geoip: map '%s': Primary GeoIP DB '%s' is not an IPv6 database and this map uses geoip_v4_overlay", map_name, db->pathname);
         geoip_db_close(db);
         return NULL;
     }
     else if((v4o_flag == V4O_SECONDARY) && db->ipv6) {
-        log_err("plugin_geoip: map '%s': geoip_v4_overlay database '%s' is not an IPv4 database", map_name, logf_pathname(db->pathname));
+        log_err("plugin_geoip: map '%s': geoip_v4_overlay database '%s' is not an IPv4 database", map_name, db->pathname);
         geoip_db_close(db);
         return NULL;
     }
@@ -635,7 +636,7 @@ static geoip_db_t* geoip_db_open(const char* pathname, const char* map_name, dcl
 nlist_t* gdgeoip_make_list(const char* pathname, const char* map_name, dclists_t* dclists, const dcmap_t* dcmap, const fips_t* fips, const gdgeoip_v4o_t v4o_flag, const bool city_auto_mode, const bool city_no_region) {
     dmn_assert(pathname); dmn_assert(map_name); dmn_assert(dclists);
 
-    log_info("plugin_geoip: map '%s': Processing GeoIP database '%s'", map_name, logf_pathname(pathname));
+    log_info("plugin_geoip: map '%s': Processing GeoIP database '%s'", map_name, pathname);
 
     nlist_t* nl = NULL;
 
