@@ -1,6 +1,6 @@
 /* Copyright © 2012 Brandon L Black <blblack@gmail.com>
  *
- * This file is part of gdnsd-plugin-geoip.
+ * This file is part of gdnsd.
  *
  * gdnsd-plugin-geoip is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,15 +23,16 @@
 #include <gdnsd/log.h>
 #include "gdmaps_test.h"
 
+static gdmaps_t* gdmaps = NULL;
+
 int main(int argc, char* argv[]) {
     if(argc != 2)
         log_fatal("root directory must be set on commandline");
 
-    gdmaps_t* gdmaps = gdmaps_test_init(argv[1]);
+    gdmaps = gdmaps_test_init(argv[1]);
     unsigned tnum = 0;
     gdmaps_test_lookup_check(tnum++, gdmaps, "my_prod_map", "::1", "\3", 101);
     gdmaps_test_lookup_check(tnum++, gdmaps, "my_prod_map", "10.0.0.0", "\1", 9);
     gdmaps_test_lookup_check(tnum++, gdmaps, "my_prod_map", "10.128.0.1", "\2", 10);
     gdmaps_test_lookup_check(tnum++, gdmaps, "my_prod_map", "FFFF:FFFF::1", "\2", 127);
-    gdmaps_destroy(gdmaps);
 }
